import configparser
import os

CONFIG = configparser.ConfigParser()

if os.path.exists('config.ini'):
    os.remove('config.ini')

IP = '10.157.1.249'
# IP = '10.10.5.108'

# CONFIG CONTENT
CONFIG['BASE'] = {}
CONFIG['BASE']['url'] = 'http://{}:8888/'.format(IP)
CONFIG['BASE']['api'] = 'api'
CONFIG['BASE']['manual'] = 'manual'


CONFIG['API'] = {}
CONFIG['API']['tcc'] = 'tcc'
CONFIG['API']['pro'] = 'pro'
CONFIG['API']['em'] = 'em'
CONFIG['API']['mm'] = 'mm'
CONFIG['API']['db'] = 'db'
CONFIG['API']['dvs'] = 'dvs'

CONFIG['RABBITMQ'] = {}
CONFIG['RABBITMQ']['host'] = IP
CONFIG['RABBITMQ']['port'] = '5672'
CONFIG['RABBITMQ']['username'] = 'admin'
CONFIG['RABBITMQ']['password'] = 'auto@T0001'


CONFIG['REDIS'] = {}
CONFIG['REDIS']['host'] = IP
CONFIG['REDIS']['password'] = 'auto@T0001'
CONFIG['REDIS']['encoding'] = 'utf-8'
CONFIG['REDIS']['db'] = '0'
CONFIG['REDIS']['max_conn'] = '20'


CONFIG['DB'] = {}
CONFIG['DB']['mysql'] = 'mysql+pymysql://root:autoTT0001@{}:3306/autotest'.format(IP)


with open('config.ini', 'w') as file:
    CONFIG.write(file)


"""
    数据库服务
"""
from ats_base.base import req, entrance
from ats_base.common import func
from ats_base.config.configure import CONFIG

db = entrance.api(CONFIG.get(func.SERVICE, 'db'))


def query(table: str, **condition):
    """
    查询
    :param table:
    :param condition:
    :return:
    """
    con = func.to_dict(condition=condition)
    result = req.post('{}/{}'.format(db, table), jsons=con)
    if result['code'] == 500:
        raise Exception(result['message'])

    if type(result['data']) is list and len(result['data']) == 1:
        return result['data'].pop()
    else:
        return result['data']


def save(table: str, **data):
    """
    保存
    :param table:
    :param data:
    :return:
    """
    td = func.to_dict(data=data)
    result = req.post('{}/{}'.format(db, table), jsons=td)
    if result['code'] == 500:
        raise Exception(result['message'])

    return result['data']


def update(table: str, condition: dict, **data):
    """
    更新
    :param table:
    :param condition:
    :param data:
    :return:
    """
    td = func.to_dict(condition=condition, data=data)
    result = req.put('{}/{}'.format(db, table), jsons=td)
    if result['code'] == 500:
        raise Exception(result['message'])

    return result['data']


def delete(table: str, **condition):
    """
    删除
    :param table:
    :param condition:
    :return:
    """
    con = func.to_dict(condition=condition)
    result = req.delete('{}/{}'.format(db, table), jsons=con)
    if result['code'] == 500:
        raise Exception(result['message'])

    return result['data']



"""
    协议栈
"""
from ats_base.base import req, entrance
from ats_base.config.configure import CONFIG
from ats_base.common import func

pro = entrance.api(CONFIG.get(func.SERVICE, 'pro'))


def manual(protocol: str, operation: str, mode: str = None, security: str = None):
    result = req.get('{}/{}'.format(pro, 'manual'),
                     params=func.to_dict(protocol=protocol, operation=operation, mode=mode, security=security))
    if result['code'] == 500:
        raise Exception(result['message'])

    return result['data']


def encode(data: dict):
    result = req.post('{}/{}'.format(pro, 'encode'), jsons=data)
    if result['code'] == 500:
        raise Exception(result['message'])

    return result['data']


def decode(data: dict):
    result = req.post('{}/{}'.format(pro, 'decode'), jsons=data)
    if result['code'] == 500:
        raise Exception(result['message'])

    return result['data']



import os
import time
import datetime
import logging
import colorlog

from logging.handlers import TimedRotatingFileHandler
from ats_base.common import func

log_colors_config = {
    'DEBUG': 'white',  # cyan white
    'INFO': 'green',
    'WARNING': 'yellow',
    'ERROR': 'red',
    'CRITICAL': 'bold_red',
}


class LogFilter:
    @staticmethod
    def info_filter(record):
        if record.levelname == 'INFO':
            return True
        return False

    @staticmethod
    def error_filter(record):
        if record.levelname == 'ERROR':
            return True
        return False


class TimeLoggerRolloverHandler(TimedRotatingFileHandler):
    def __init__(self, filename, when='h', interval=1, backupCount=0, encoding=None, delay=False, utc=False,
                 atTime=None):
        super(TimeLoggerRolloverHandler, self).__init__(filename, when, interval, backupCount, encoding, delay, utc)

    def doRollover(self):
        """
        """
        if self.stream:
            self.stream.close()
            self.stream = None
        currentTime = int(time.time())
        dstNow = time.localtime(currentTime)[-1]
        log_type = 'info' if self.level == 20 else 'error'
        dfn = f"test_{datetime.datetime.now().strftime('%Y-%m-%d')}.{log_type}.log"
        self.baseFilename = dfn
        if not self.delay:
            self.stream = self._open()
        newRolloverAt = self.computeRollover(currentTime)
        while newRolloverAt <= currentTime:
            newRolloverAt = newRolloverAt + self.interval
        if (self.when == 'MIDNIGHT' or self.when.startswith('W')) and not self.utc:
            dstAtRollover = time.localtime(newRolloverAt)[-1]
            if dstNow != dstAtRollover:
                if not dstNow:
                    addend = -3600
                else:
                    addend = 3600
                newRolloverAt += addend
        self.rolloverAt = newRolloverAt


# 定义收集logger的级别
logger = logging.getLogger()
logger.setLevel(logging.INFO)

# 定义handler的输出格式
fmt = logging.Formatter("[%(asctime)s] - %(levelname)s: %(message)s")
color_fmt = colorlog.ColoredFormatter("[%(asctime)s] - %(levelname)s: %(message)s", log_colors=log_colors_config)

# 日志文件的handler
log_error_file = 'test_{}.error.log'.format(datetime.datetime.now().strftime('%Y-%m-%d'))
log_info_file = 'test_{}.info.log'.format(datetime.datetime.now().strftime('%Y-%m-%d'))
report_dir = func.makeDir(func.project_dir(), 'hist')

error_handler = TimeLoggerRolloverHandler(report_dir + os.sep + log_error_file
                                          , when='midnight', backupCount=7, encoding='utf8')
error_handler.addFilter(LogFilter.error_filter)
error_handler.setLevel(logging.ERROR)

info_handel = TimeLoggerRolloverHandler(report_dir + os.sep + log_info_file
                                        , when='midnight', backupCount=7, encoding='utf8')
info_handel.addFilter(LogFilter.info_filter)
info_handel.setLevel(logging.INFO)

# 配置控制台的handler
console_handler = logging.StreamHandler()
console_handler.setLevel(logging.INFO)

console_handler.setFormatter(color_fmt)
info_handel.setFormatter(fmt)
error_handler.setFormatter(fmt)

# 将logger添加到handler里面
logger.addHandler(console_handler)
logger.addHandler(info_handel)
logger.addHandler(error_handler)




"""
    加密机
"""
from ats_base.base import req, entrance
from ats_base.config.configure import CONFIG
from ats_base.common import func

em = entrance.api(CONFIG.get(func.SERVICE, 'em'))


def handle(protocol: str, function: str, data):
    result = req.post('{}/{}/{}'.format(em, protocol, function), jsons=data)
    if result['code'] == 500:
        raise Exception(result['message'])

    return result['data']


OK = 'OK'
NULL = 'NULL'
ERROR = '{api}-ERROR: {msg}.'

SUCCESS = 200
FAILURE = 500


def success(data=None):
    """
    系统响应消息
    :param code:
    :param msg:
    :param data:
    :return:
    """
    return info(SUCCESS, OK, data)


def failure(msg: str, data=None):
    """
    系统响应消息
    :param code:
    :param msg:
    :param data:
    :return:
    """
    return info(FAILURE, msg, data)


def info(code: int, msg: str, data):
    """
    系统响应消息
    :param code:
    :param msg:
    :param data:
    :return:
    """
    return {'code': code, 'message': msg, 'data': data}



"""
    结果分析计算验证服务
"""
from ats_base.base import req, entrance
from ats_base.common import func
from ats_base.config.configure import CONFIG

udm = entrance.api(CONFIG.get(func.SERVICE, 'udm'))


def handle(module: str, function: str, data, debug_url: str = None):
    """
    数据验证
    :param module:
    :param function:
    :param data:
    :param debug_url:
    :return:
    """
    if debug_url is not None and func.is_valid_url(debug_url):
        result = req.post('{}/{}/{}'.format(debug_url, module, function), jsons=data)
    else:
        result = req.post('{}/{}/{}'.format(udm, module, function), jsons=data)

    if result['code'] == 500:
        raise Exception(result['message'])

    return result['data']



from ats_base.base import req
from ats_base.config.configure import CONFIG


api_entrance = CONFIG.get('BASE', 'url') + CONFIG.get('BASE', 'api')
manual_entrance = CONFIG.get('BASE', 'url') + CONFIG.get('BASE', 'manual')


def api(service: str):
    """
    获取服务api访问url地址
    :param service:
    :return:
    """
    url = '{}/{}'.format(api_entrance, service)

    result = req.get(url)
    if result['code'] == 500:
        raise Exception(result['message'])

    return result['data']


def manual(name: str, key: str):
    """
    获取 - 参数格式
    :param name:
    :param key:
    :param params:
    :return:
    """
    url = '{}/{}/{}'.format(manual_entrance, name, key)

    result = req.get(url)
    if result['code'] == 500:
        raise Exception(result['message'])

    return result['data']



import configparser
import os

CONFIG = configparser.ConfigParser()

cur_path = os.path.dirname(os.path.realpath(__file__))
config_path = os.path.join(cur_path, 'config.ini')

if os.path.exists(config_path):
    CONFIG.read(config_path)





"""
    缓存服务 - redis
    - 缓存测试用例运行过程中产生的数据 - 方便调用
"""
from ats_base.base import req, entrance
from ats_base.common import func
from ats_base.config.configure import CONFIG

mm = entrance.api(CONFIG.get(func.SERVICE, 'mm'))


class Dict:
    """
        字典
    """
    url = mm + '/dict'

    @classmethod
    def get(cls, name: str, key: str):
        result = req.get('{}/{}/{}'.format(Dict.url, name, key))
        if result['code'] == 500:
            raise Exception(result['message'])

        return result['data']

    @classmethod
    def put(cls, name: str, key: str, data):
        if type(data) is dict or type(data) is list:
            result = req.post('{}/{}/{}'.format(Dict.url, name, key), jsons=data)
        else:
            result = req.post('{}/{}/{}/{}'.format(Dict.url, name, key, data))

        if result['code'] == 500:
            raise Exception(result['message'])

        return result['data']

    @classmethod
    def delete(cls, name: str, key: str):
        result = req.delete('{}/{}/{}'.format(Dict.url, name, key))
        if result['code'] == 500:
            raise Exception(result['message'])

        return result['message']


class Queue:
    """
        队列
    """
    url = mm + '/queue'

    @classmethod
    def get(cls, name: str):
        result = req.get('{}/{}'.format(Queue.url, name))
        if result['code'] == 500:
            raise Exception(result['message'])

        return result['data']

    @classmethod
    def put(cls, name: str, data):
        if type(data) is dict or type(data) is list:
            result = req.post('{}/{}'.format(Queue.url, name), jsons=data)
        else:
            result = req.post('{}/{}/{}'.format(Queue.url, name, data))

        if result['code'] == 500:
            raise Exception(result['message'])

        return result['data']

    @classmethod
    def delete(cls, name: str, start: int, end: int):
        result = req.delete('{}/{}/{}/{}'.format(Dict.url, name, start, end))
        if result['code'] == 500:
            raise Exception(result['message'])

        return result['message']


class Set:
    """
        集合
    """
    url = mm + '/set'

    @classmethod
    def get(cls, name: str):
        result = req.get('{}/{}'.format(Set.url, name))
        if result['code'] == 500:
            raise Exception(result['message'])

        return result['data']

    @classmethod
    def put(cls, name: str, data):
        result = req.post('{}/{}/{}'.format(Set.url, name, data))
        if result['code'] == 500:
            raise Exception(result['message'])

        return result['message']

    @classmethod
    def delete(cls, name: str, data):
        result = req.delete('{}/{}/{}'.format(Set.url, name, data))
        if result['code'] == 500:
            raise Exception(result['message'])

        return result['message']


def get(key: str):
    result = req.get('{}/{}'.format(mm, key))
    if result['code'] == 500:
        raise Exception(result['message'])

    return result['data']


def put(key: str, data):
    if type(data) is dict or type(data) is list:
        result = req.post('{}/{}'.format(mm, key), jsons=data)
    else:
        result = req.post('{}/{}/{}'.format(mm, key, data))
        
    if result['code'] == 500:
        raise Exception(result['message'])

    return result['message']


def delete(key: str):
    result = req.delete('{}/{}'.format(mm, key))
    if result['code'] == 500:
        raise Exception(result['message'])

    return result['message']


if __name__ == '__main__':
    METER_COMM = {
        "step_id": "*STEP_ID",
        "todo": {
            "meter:comm": {
                "msg": "#MESSAGE",
                "channel": '#CHANNEL',
                "frame": "#FRAME"
            }
        }
    }

    APP_Show = {
        "step_id": "*STEP_ID",
        "todo": {
            "app:show": {
                "msg": "#MESSAGE"
            }
        }
    }

    BENCH_POWER_OFF = {
        "step_id": "*STEP_ID",
        "todo": {
            "bench:power_off": {
                "msg": "#MESSAGE",
                "Dev_Port": "#DEV_PORT"
            }
        }
    }

    BENCH_POWER_ON = {
        "step_id": "*STEP_ID",
        "todo": {
            "bench:power_on": {
                "msg": "#MESSAGE",
                "Phase": "#PHASE",
                "Rated_Volt": "#RATED_VOLTAGE",
                "Rated_Curr": "#RATED_CURRENT",
                "Rated_Freq": "#RATED_FREQUENT",
                "PhaseSequence": "#PH_SEQUENCE",
                "Revers": "#REVERS",
                "Volt_Per": "#VOLTAGE_PERCENT",
                "Curr_Per": "#CURRENT_PERCENT",
                "IABC": "#IABC",
                "CosP": "#COSP",
                "SModel": "#SMODEL",
                "Dev_Port": "#DEV_PORT"
            }
        }
    }

    BENCH_ADJUST = {
        "step_id": "*STEP_ID",
        "todo": {
            "bench:adjust": {
                "msg": "#MESSAGE",
                "Phase": "#PHASE",
                "Rated_Volt": "#RATED_VOLTAGE",
                "Rated_Curr": "#RATED_CURRENT",
                "Rated_Freq": "#RATED_FREQUENT",
                "PhaseSequence": "#PH_SEQUENCE",
                "Revers": "#REVERS",
                "Volt_Per": "#VOLTAGE_PERCENT",
                "Curr_Per": "#CURRENT_PERCENT",
                "IABC": "#IABC",
                "CosP": "#COSP",
                "SModel": "#SMODEL",
                "Dev_Port": "#DEV_PORT"
            }
        }
    }

    BENCH_ADJUST_CUST = {
        "step_id": "*STEP_ID",
        "todo": {
            "bench:adjust_cust": {
                "msg": "#MESSAGE",
                "Phase": "#PHASE",
                "Rated_Freq": "#RATED_FREQUENT",
                "Volt1": "#VOLTAGE1",
                "Volt2": "#VOLTAGE2",
                "Volt3": "#VOLTAGE3",
                "Curr1": "#CURRENT1",
                "Curr2": "#CURRENT2",
                "Curr3": "#CURRENT3",
                "Uab": "#UAB",
                "Uac": "#UAC",
                "Ang1": "#ANGLE1",
                "Ang2": "#ANGLE2",
                "Ang3": "#ANGLE3",
                "SModel": "#SMODEL",
                "Dev_Port": "#DEV_PORT"
            }
        }
    }

    BENCH_ADJUST_UI2 = {
        "step_id": "*STEP_ID",
        "todo": {
            "bench:adjust_ui2": {
                "msg": "#MESSAGE",
                "Phase": "#PHASE",
                "Rated_Volt": "#RATED_VOLTAGE",
                "Rated_Curr": "#RATED_CURRENT",
                "Rated_Freq": "#RATED_FREQUENT",
                "PhaseSequence": "#PHASE_SEQUENCE",
                "Revers": "#REVERS",
                "Volt1_Per": "#VOLTAGE1_PERCENT",
                "Volt2_Per": "#VOLTAGE2_PERCENT",
                "Volt3_Per": "#VOLTAGE3_PERCENT",
                "Curr1_Per": "#CURRENT1_PERCENT",
                "Curr2_Per": "#CURRENT2_PERCENT",
                "Curr3_Per": "#CURRENT3_PERCENT",
                "IABC": "#IABC",
                "CosP": "#COSP",
                "SModel": "#SMODEL",
                "Dev_Port": "#DEV_PORT"
            }
        }
    }

    BENCH_READ = {
        "step_id": "*STEP_ID",
        "todo": {
            "bench:read": {
                "msg": "#MESSAGE",
                "SModel": "#SMODEL",
                "Dev_Port": "#DEV_PORT"
            }
        }
    }

    Dict.put('app:manual', 'meter:comm', METER_COMM)
    Dict.put('app:manual', 'app:show', APP_Show)
    Dict.put('app:manual', 'bench:power_on', BENCH_POWER_ON)
    Dict.put('app:manual', 'bench:adjust', BENCH_ADJUST)
    Dict.put('app:manual', 'bench:adjust_cust', BENCH_ADJUST_CUST)
    Dict.put('app:manual', 'bench:adjust_ui2', BENCH_ADJUST_UI2)
    Dict.put('app:manual', 'bench:power_off', BENCH_POWER_OFF)
    Dict.put('app:manual', 'bench:read', BENCH_READ)

    # API Service URL
    Dict.put("API", 'TCC', 'http://10.10.5.108:8000/tcc')
    Dict.put("API", 'PRO', 'http://10.10.5.108:8001/pro')
    Dict.put("API", 'MM', 'http://10.10.5.108:8007/mm')
    Dict.put("API", 'DB', 'http://10.10.5.108:8008/db')
    Dict.put("API", 'DVS', 'http://10.10.5.108:8020/dvs')
    Dict.put("API", 'GW698', 'http://10.10.5.108:8010/pro/gw698')
    Dict.put("API", 'DLMS', 'http://10.10.5.108:8011/pro/dlms')
    Dict.put("API", 'DLT645', 'http://10.10.5.108:8012/pro/dlt645')

    # client.getConn().hset("MANUAL:API", 'TCC', {})



import os
import re
import time
import uuid

from pathlib import Path

from dateutil import parser
from dateutil.relativedelta import relativedelta

SERVICE = 'API'


def project_dir():
    root = os.getenv('PROJECT_DIR')
    if root is None:
        return '/service/app'

    return root


def to_dict(**kwargs):
    """
    转化为dict类型数据
    :param kwargs:
    :return:
    """
    return kwargs


def to_list(*args):
    """
    转化为list类型数据
    :param args:
    :return:
    """
    return args


def extract_digit(s: str):
    """
    提取字符串中所有数字
    :param s:
    :return:
    """
    return "".join(list(filter(str.isdigit, s)))


def time_add(dt: str, years: int = 0, months: int = 0, days: int = 0,
             hours: int = 0, minutes: int = 0, seconds: int = 0):
    """
    时间加法器
    :param dt:
    :param years:
    :param months:
    :param days:
    :param hours:
    :param minutes:
    :param seconds:
    :return:
    """
    t = parser.parse(dt)
    t = t + relativedelta(years=years, months=months, days=days, hours=hours, minutes=minutes, seconds=seconds)

    return t.strftime('%Y-%m-%d %H:%M:%S')


def time_diff(dt1: str, dt2: str):
    """
    时间差
    :param dt1:
    :param dt2:
    :return: 秒
    """
    t1 = parser.parse(dt1)
    t2 = parser.parse(dt2)

    return (t2 - t1).total_seconds()


def is_valid_url(url: str):
    """
    验证url有效性
    :param url:
    :return:
    """
    regex = r'http[s]?://(?:[a-zA-Z][0-9]|[$-@.&+]|[!*\(\),]|(?:%[0-9a-zA-F]))+'

    p = re.compile(regex)

    if url is None:
        return False

    if re.search(p, url):
        return True

    return False


def replace(content: dict, **kwargs):
    """
    替换字典中变量
    :param content:
    :return:
    """
    ds = str(content)

    for key, value in kwargs.items():
        if type(value) is str:
            ds = ds.replace('#{}'.format(key.upper()), value.replace('\'', '"'))
            ds = ds.replace('*{}'.format(key.upper()), value.replace('\'', '"'))
        else:
            ds = ds.replace('\'#{}\''.format(key.upper()), str(value))
            ds = ds.replace('\'*{}\''.format(key.upper()), str(value))

    return eval(ds)


def gen_sn():
    """
    生成唯一的测试序列号
    :return:
    """
    return uuid.uuid4().hex


def sys_current_time():
    """
    服务器系统时间
    :return:
    """
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())


def tcp_extract(tcp: str):
    """
    TCP获取ip和port
    :param tcp:
    :return:
    """
    result = re.search("((?<![\.\d])(?:\d{1,3}\.){3}\d{1,3}(?![\.\d]))", tcp)
    if result:
        return result.group(1), int(tcp.split(':')[2])
    else:
        return "tcp cannot find ip or port"


def exist(project_path, *args):
    """
    目录是否存在
    :param project_path:
    :param args:
    :return:
    """
    file_path = os.path.join(project_path, *args)
    if os.path.exists(file_path):
        return True
    else:
        return False


def makeDir(project_path, *args):
    """
    创建目录
    :param project_path:
    :param args:
    :return:
    """
    dir_path = os.path.join(project_path, *args)
    if not exist(project_path, *args):
        Path(dir_path).mkdir(parents=True, exist_ok=True)

    return dir_path


if __name__ == '__main__':
    print(time.localtime())
    s = time.localtime()
    import datetime

    someday = datetime.datetime.now()
    print(someday.strftime("%Y%m%d%H%M%S%f"))
    print(time.strftime("%Y%m%d%H%M%S", time.localtime()))



import requests
import json

_timeout_ = 90
_method_ = 'get'


def get(url, params: dict = None):
    """
    请求方式 - get
    :param url:
    :param params:
    :return:
    """
    try:
        res = requests.get(url=url, params=params, timeout=_timeout_)
        try:
            return json.loads(res.text)
        except Exception as e:
            return res.text
    except Exception as e:
        raise e


def post(url, params: dict = None, jsons: dict = None):
    """
    请求方式 - post
    :param url:
    :param params:
    :param jsons:
    :return:
    """
    try:
        res = requests.post(url=url, params=params, json=jsons, timeout=_timeout_)
        try:
            return json.loads(res.text)
        except Exception as e:
            return res.text
    except Exception as e:
        raise e


def put(url, params: dict = None, jsons: dict = None):
    """
    请求方式 - put
    :param url:
    :param params:
    :param jsons:
    :return:
    """
    try:
        res = requests.put(url=url, params=params, json=jsons, timeout=_timeout_)
        try:
            return json.loads(res.text)
        except Exception as e:
            return res.text
    except Exception as e:
        raise e


def delete(url, params: dict = None, jsons: dict = None):
    """
    请求方式 - delete
    :param url:
    :param params:
    :param jsons:
    :return:
    """
    try:
        res = requests.delete(url=url, params=params, json=jsons, timeout=_timeout_)
        try:
            return json.loads(res.text)
        except Exception as e:
            return res.text
    except Exception as e:
        raise e




